/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.platform.raspberry.ui.compose;

import static org.fortiss.af3.platform.utils.PlatformArchitectureUtils.isMasterPort;
import static org.fortiss.af3.platform.utils.PlatformArchitectureUtils.isPlatformElement;

import org.eclipse.emf.common.util.EList;
import org.fortiss.af3.platform.compose.base.hierarchical.elements.PlatformElementCompositorBase;
import org.fortiss.af3.platform.model.GenericPlatformUnit;
import org.fortiss.af3.platform.model.IArchitectureDomain;

/**
 * 
 * @author chaudhary
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash:
 */
public abstract class RaspberryPi1CompositorBase<T extends GenericPlatformUnit & IArchitectureDomain>
		extends PlatformElementCompositorBase<T> {

	/** {@inheritDoc} */
	@Override
	protected boolean canBeAddedAtCurrentLevel(IArchitectureDomain element) {
		return isMasterPort(element);
	}

	/** {@inheritDoc} */
	@Override
	protected boolean canBeDroppedInto(IArchitectureDomain element) {
		EList<Class<? extends IArchitectureDomain>> childDomains =
				getPlatformCompositionRules().getAdmissibleChildDomains(getEditedObjectClass());
		if(childDomains.isEmpty()) {
			return isPlatformElement(element);
		}
		for(Class<? extends IArchitectureDomain> domain : childDomains) {
			if(domain.isAssignableFrom(element.getClass())) {
				return isPlatformElement(element);
			}
		}

		return false;
	}

}
