/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.platform.raspberry.ui.compose.connections;

import static org.fortiss.af3.platform.raspberry.utils.RaspberryModelElementFactory.createBusMasterInterface;
import static org.fortiss.af3.platform.raspberry.utils.RaspberryModelElementFactory.createBusMasterPort;
import static org.fortiss.af3.platform.utils.PlatformModelElementFactory.createTransmissionConnection;

import org.fortiss.af3.platform.model.GenericPlatformUnit;
import org.fortiss.af3.platform.raspberry.model.Bus;
import org.fortiss.af3.platform.raspberry.model.Core;
import org.fortiss.tooling.base.compose.HierarchicElementConnectionCompositorBase;
import org.fortiss.tooling.base.model.element.IConnection;
import org.fortiss.tooling.base.model.element.IConnector;
import org.fortiss.tooling.kernel.extension.data.IConnectionCompositionContext;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.service.IPersistencyService;

/**
 * 
 * @author chaudhary
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash:
 */
public class CoreToBusConnectionCompositor extends
		HierarchicElementConnectionCompositorBase<GenericPlatformUnit, Core, Bus> {

	/** {@inheritDoc} */
	@Override
	public boolean canConnect(Core source, Bus target, GenericPlatformUnit parent,
			IConnectionCompositionContext context) {

		if(source.eContainer() != target.eContainer()) {
			return false;
		}
		return super.canConnect(source, target, parent, context);
	}

	/** {@inheritDoc} */
	@Override
	public boolean connect(Core source, Bus target, GenericPlatformUnit parent,
			IConnectionCompositionContext context) {
		IConnector sourceConnector = createEntryConnector();
		avoidDuplicateConnectorName(source, sourceConnector);

		IConnector targetConnector = createExitConnector();
		avoidDuplicateConnectorName(target, targetConnector);

		ITopLevelElement topElement = IPersistencyService.INSTANCE.getTopLevelElementFor(parent);
		if(topElement != null) {
			topElement.prepareIDs(sourceConnector);
			topElement.prepareIDs(targetConnector);
		}
		createNewSourceDropContext(source, context, sourceConnector);
		createNewTargetDropContext(target, context, targetConnector);

		createConnection(sourceConnector, targetConnector, parent, topElement);
		return true;
	}

	/** {@inheritDoc} */
	@Override
	protected IConnector createEntryConnector() {
		return createBusMasterPort("BusMasterPort");
	}

	/** {@inheritDoc} */
	@Override
	protected IConnector createExitConnector() {
		return createBusMasterInterface("BusMasterInterface");
	}

	/** {@inheritDoc} */
	@Override
	protected IConnection createConnection() {
		return createTransmissionConnection();
	}

}
